<?php
class GCFClass {
	var $header_dummy1 = 0;
	var $header_dummy2 = 0;
	var $gcfversion = 0;
	var $appid = 0;
	var $appversion = 0;
	var $header_dummy3 = 0;
	var $header_dummy4 = 0;
	var $filesize = 0;
	var $blocksize = 0;
	var $blockcount = 0;
	var $header_dummy5 = 0;
	var $firstblockoffset = 0;

	var $block_header = array();
	var $block_entries = array();
	var $frag_entries = array();
	var $dirmap_entries = array();

	var $manifest = array();
	var $checksums = array();

	var $checksumsize = 0;

	var $manifest_index = 0;
	var $dirmap_index = 0;
	var $checksum_index = 0;

	var $f;

	function Load($filepath) {
		$this->f = fopen($filepath, "r");
		$this->Initialize();
	}

	function Initialize() {
		$header = fread($this->f, 44);
		list(,$this->header_dummy1,
		$this->header_dummy2,
		$this->gcfversion,
		$this->appid,
		$this->appversion,
		$this->header_dummy3,
		$this->header_dummy4,
		$this->filesize,
		$this->blocksize,
		$this->blockcount,
		$this->header_dummy5) = unpack("L11", $header);

		if (!($this->gcfversion != 6 || $this->gcfversion != 1)) {
			$this->error("Error, File Version Not Allowed: ".$this->gcfversion);
		}

		if ($this->gcfversion == 6) {
            	list(,$blockcount,$blocksused,$dummy0,$dummy1,$dummy2,$dummy3,$dummy4,$checksum) = unpack("L8", fread($this->f, 32));
			if ($blockcount != $this->blockcount) $this->error("Different block counts in header and block header: ".$blockcount." ".$blockcount);
			if ($dummy1 != 0 or $dummy2 != 0 or $dummy3 != 0 or $dummy4 != 0) $this->error("Dummy values different than expected: ".$dummy1." ".$dummy2." ".$dummy3." ".$dummy4);

			for ($i=0;$i < $this->blockcount;$i++) {
				$b = array();
				list(,$b['entry_type'], $b['file_data_offset'], $b['file_data_size'], $b['first_data_block_index'], $b['next_block_entry_index'], $b['prev_block_entry_index'], $b['dir_index']) = unpack("L7", fread($this->f, 28));
				$this->block_entries[$i] = $b;
			}
			unset($b);

			$frag_header = fread($this->f, 16);
			list(,$blockcount, $dummy0, $dummy1, $checksum) = unpack("L4", $frag_header);
			if ($blockcount != $this->blockcount) $this->error("Different block counts in header and frag header: ".$this->blockcount." ".$blockcount);
			unset($frag_header);
			for ($i=0;$i < $this->blockcount;$i++) list(,$this->frag_entries[$i]) = unpack("L", fread($this->f, 4));
		}

		$this->manifest_index = ftell($this->f);
		$manifest_header = fread($this->f, 56);
		list(,$manifest_size) = unpack("L", substr($manifest_header, 24, 4));
		$manifest_binary = fread($this->f, $manifest_size - 56);
		$manifest_data = $manifest_header.$manifest_binary;
		$this->manifest = $this->LoadManifest($manifest_data);
		fseek($this->f, $this->manifest_index + $manifest_size);
		unset($manifest_header);
		unset($manifest_binary);
		unset($manifest_data);

		$this->dirmap_index = ftell($this->f);
		$dirmap_header = fread($this->f, 8);
		for ($i=0;$i < $this->manifest['num_items'];$i++) {
			list(,$dirmap_entry) = unpack("L1", fread($this->f, 4));
			$this->dirmap_entries[$i] = $dirmap_entry;
		}
		unset($dirmap_header);

		$this->checksum_index = ftell($this->f);
		$checksum_header = fread($this->f, 8);
		list(,$dummy0, $this->checksumsize) = unpack("L2", $checksum_header);
    		$checksum_data = fread($this->f, $this->checksumsize);
		$this->checksums = $this->LoadChecksums($checksum_data);
		unset($checksum_header);
		unset($checksum_data);

		if ($this->gcfversion == 6) {
			$block_header = fread($this->f, 24);
			list(,$this->block_header['appversion'],
			$this->block_header['blockcount'],
			$this->block_header['blocksize'],
			$this->firstblockoffset,
			$this->block_header['blocksused'],
			$this->block_header['checksum']) = unpack("L6", $block_header);
			unset($block_header);
		}
		fseek($this->f, 0);
	}

	function LoadManifest($manifest_data, $loadfromfile = "") {
		$self = array();
		if ($manifest_data == "") {
			$file = fopen($loadfromfile, "r");
			$manifest_data = fread($file, filesize($loadfromfile));
			fclose($file);
		}
		list(,$self['dummy1'],
		$self['stored_appid'],
		$self['stored_appver'],
		$self['num_items'],
		$self['num_files'],
		$self['blocksize'],
		$self['dirsize'],
		$self['dirnamesize'],
		$self['info1count'],
		$self['copycount'],
		$self['localcount'],
		$self['dummy2'],
		$self['dummy3'],
		$self['checksum']) = unpack("L14", substr($manifest_data, 0, 56));
		$self['dirnames_start'] = (56 + $self['num_items'] * 28);
		$self['dir_entries1'] = array();
		$self['dir_entries2'] = array();

		if (!$self['num_items']) error("Error in LoadManifest(), num items: ".$self['num_items']);

		for ($i=0;$i < $self['num_items'];$i++) {
           		$index = 56 + $i * 28;
           		$dir_entry = array();
			$dir_entry['i'] = $i;
           		list(,$dir_entry['nameoffset'],
			$dir_entry['itemsize'],
			$dir_entry['fileid'],
			$dir_entry['dirtype'],
			$dir_entry['parentindex'],
			$dir_entry['nextindex'],
			$dir_entry['firstindex']) = unpack("L7", substr($manifest_data, $index, 28));
           		$filename_start = $self['dirnames_start'] + $dir_entry['nameoffset'];
           		$filename_end = strpos($manifest_data, "\x00", $filename_start);
           		$dir_entry['filename'] = substr($manifest_data, $filename_start, $filename_end-$filename_start);
			$self['dir_entries1'][$i] = $dir_entry;
			unset($dir_entry);
		}
		unset($manifest_data);

		for ($i=0;$i < $self['num_items'];$i++) {
			$dir_entry = $self['dir_entries1'][$i];
			$fullfilename = $dir_entry['filename'];
           		while ($dir_entry['parentindex'] != "") {
				$dir_entry = $self['dir_entries1'][$dir_entry['parentindex']];
				$fullfilename = $dir_entry['filename']."/".$fullfilename;
			}
			$self['dir_entries1'][$i]['fullfilename'] = $fullfilename;
		}

		for ($i=0;$i < $self['num_items'];$i++) {
			$dir_entry = $self['dir_entries1'][$i];
			$self['dir_entries2'][$dir_entry['fileid']] = $dir_entry;
		}

		return $self;
	}

	function LoadChecksums($checksum_data, $loadfromfile = "") {
		$self = array();
		if ($checksum_data == "") {
			$file = fopen($loadfromfile, "r");
			$checksum_data = fread($file, filesize($loadfromfile));
			fclose($file);
		}
		list(,$dummy,$dummy2,$self['numfiles'],$self['totalchecksums']) = unpack("L4", substr($checksum_data, 0, 16));
		$self['checksumliststart'] = $self['numfiles']* 8 + 16;

		$self['numchecksums'] = array();
		$self['checksumstart'] = array();
		$self['checksums'] = array();
		$self['checksums_raw'] = array();

		for ($fileid=0;$fileid < $self['numfiles'];$fileid++) {
			$checksumpointer = $fileid * 8 + 16;
			list(,$self['numchecksums'][$fileid], $self['checksumstart'][$fileid]) = unpack("L2", substr($checksum_data, $checksumpointer, 8));
			$filechecksums = array();
			$start = $self['checksumliststart'] + $self['checksumstart'][$fileid] * 4;
			$len   = $self['numchecksums'][$fileid] * 4;
			$self['checksums_raw'][$fileid] = substr($checksum_data, $start, $len);
			for ($i=0;$i < $self['numchecksums'][$fileid];$i++) {
				list(,$filechecksums[$i]) = unpack("i", substr($checksum_data, $start, 4));
				$start += 4;
			}
			$self['checksums'][$fileid] = $filechecksums;
			unset($filechecksums);
		}
		unset($checksum_data);
		return $self;
	}

	function close() {
		fclose($this->f);

		unset($this->header_dummy1);
		unset($this->header_dummy2);
		unset($this->gcfversion);
		unset($this->appid);
		unset($this->appversion);
		unset($this->header_dummy3);
		unset($this->header_dummy4);
		unset($this->filesize);
		unset($this->blocksize);
		unset($this->blockcount);
		unset($this->header_dummy5);
		unset($this->firstblockoffset);

		unset($this->block_header);
		unset($this->block_entries);
		unset($this->frag_entries);
		unset($this->dirmap_entries);

		unset($this->manifest);
		unset($this->checksums);

		unset($this->checksumsize);

		unset($this->manifest_index);
		unset($this->dirmap_index);
		unset($this->checksum_index);

		$this->header_dummy1 = 0;
		$this->header_dummy2 = 0;
		$this->gcfversion = 0;
		$this->appid = 0;
		$this->appversion = 0;
		$this->header_dummy3 = 0;
		$this->header_dummy4 = 0;
		$this->filesize = 0;
		$this->blocksize = 0;
		$this->blockcount = 0;
		$this->header_dummy5 = 0;
		$this->firstblockoffset = 0;

		$this->block_header = array();
		$this->block_entries = array();
		$this->frag_entries = array();
		$this->dirmap_entries = array();

		$this->manifest = array();
		$this->checksums = array();

		$this->checksumsize = 0;

		$this->manifest_index = 0;
		$this->dirmap_index = 0;
		$this->checksum_index = 0;
	}

	function error($text, $close = true) {
		if ($close == true) $this->close();
		die($text);
	}
}
?>