<?php
class STEAMClass {
	var $directory_servers = Array(
		Array("72.165.61.189", 27030),
		Array("72.165.61.190", 27030),
		Array("69.28.151.178", 27038),
		Array("69.28.153.82", 27038),
		Array("68.142.88.34", 27038),
		Array("68.142.72.250", 27038));

	/*---------------------------------------------------------------------------*/
	// "\x1c"			 - auth servers(?)
	// "\x12"			 - auth servers(?)
	// "\x00", "userhash"	 - auth servers with a specific username
	// "\x03"			 - config servers
	// "\x06"			 - content list servers
	// "\x14"			 - CSER server(s)
	// "\x0f"			 - HL1 Master servers
	// "\x18"			 - HL2 Master servers
	// "\x1e"			 - Rag Doll Kung Fu Master servers
	/*---------------------------------------------------------------------------*/
	function steam_directory_client($command, $userhash = "") {
		$rnd = rand(0, count($this->directory_servers)-1);
		$address = $this->directory_servers[$rnd];
		$socket = New ImpSocket;
		$addresses = array();
		if($socket->connect($address)) {
			$socket->send("\x00\x00\x00\x02");
			$reply = $socket->recv(1);
			$addrs = "";
			if ($reply == "\x01") {
				$socket->send_withlen($command.$userhash);
				$addrs = $socket->recv_withlen();
			}
			$socket->close();
			if ($addrs != "") {
				list(,$addrs_num) = unpack("v", $addrs[1].$addrs[0]);
				for($i=0;$i < $addrs_num;$i++) {
					$start = $i * 6 + 2;
					$server = $this->decodeip(substr($reply, $start, 6));
					$addresses[] = $server;
				}
			}
		} else {
			//  
		}
		return $addresses;
	}

	function steam_download_cdr($address, $decompress = 0, $old_cdr_checksum = "") {
		$cdr_bin = ""; $checksum = "";
		$socket = New ImpSocket;
		if($socket->connect($address)) {
			$socket->send("\x00\x00\x00\x03");
			$reply = $socket->recv(5);
			//$ip = ord($reply[4]).".".ord($reply[3]).".".ord($reply[2]).".".ord($reply[1]);
			if ($old_cdr_checksum != "") {
    				$old_cdr_checksum = pack("H40", $old_cdr_checksum);
			} else {
				$old_cdr_checksum = pack("H40", sha1(""));
			}
			$socket->send_withlen("\x02".$old_cdr_checksum);
			$cdr_bin = $socket->recv_withlen();
			$socket->close();
			$checksum = sha1($cdr_bin);
			if ($decompress == 1) {
				$cdr_bin = gzuncompress(substr($cdr_bin, 20));
			}
		} else {
			$cdr_bin = False;
		}
		return array($cdr_bin, $checksum);
	}

	function blob_unserialize($blobtext) {
		$blobdict = array();
		$blob_info = unpack('Ltotalsize/Lslack', substr($blobtext, 2, 8));

		if ($blob_info['slack']) {
			$blobdict['__slack__'] = substr($blobtext, $blob_info['slack']);
		}

		if (($blob_info['totalsize'] + $blob_info['slack']) != strlen($blobtext)) {
			die("Blob not correct length including slack space!".$blob_info['totalsize']);
		}

		$index = 10;
		while ($index < $blob_info['totalsize']) {
			$namestart = $index + 6;
			$subblob_info = unpack('Snamesize/Ldatasize', substr($blobtext, $index, 6));
			$datastart = $namestart + $subblob_info['namesize'];
			$name = substr($blobtext, $namestart, $subblob_info['namesize']);
			$data = substr($blobtext, $datastart, $subblob_info['datasize']);
			if (strlen($data) > 2 && $data[0] == "\x01" && $data[1] == "\x50") {
				$sub_blob = $this->blob_unserialize($data);
				$blobdict[$name] = $sub_blob;
			} else {
				$blobdict[$name] = $data;
			}
			$index += $subblob_info['namesize'] + $subblob_info['datasize'] + 6;
		}
		return $blobdict;
	}

	function blob_serialize($blobdict) {
		$blobtext = "";
		foreach ($blobdict as $name => $data) {
			if ($name == "__slack__") {
				continue;
			}
			if (is_array($data)) {
				$data = $this->blob_serialize($data);
			}
			$namesize = strlen($name);
			$datasize = strlen($data);
			$subtext = pack("vL", $namesize, $datasize);
			$subtext = $subtext.$name.$data;
			$blobtext = $blobtext.$subtext;
		}
		if (in_array("__slack__", $blobdict)) {
			$slack = $blobdict["__slack__"];
		} else {
			$slack = "";
		}
		$totalsize = strlen($blobtext) + 10;
		$sizetext = pack("L2", $totalsize, strlen($slack));
		$blobtext = chr(0x01).chr(0x50).$sizetext.$blobtext.$slack;
		return $blobtext;
	}

	function blob_dump($blobdict, $spacer = "") {
		$text = $spacer."Array(\n";
		$first = True;
		foreach ($blobdict as $name => $data) {
			$spacer2 = $spacer."	";
			if (is_array($data)) {
				if (!$first) {
					$text .= ",";
				} else {
					$first = False;
				}
				$text .= "\n".$spacer2.$this->formatstring($name)." => \n".$this->blob_dump($data, $spacer2);
			} else {
				if (!$first) {
					$text .= ",";
				} else {
					$first = False;
				}
				$text .= "\n".$spacer2.$this->formatstring($name)." => ".$this->formatstring($data);
			}
		}
		$text .= "\n".$spacer.")";
		if ($spacer == "") $text .= ";";
		return $text;
	}

	function decodeIP($string) {
		list(, $port) = unpack("v", substr($string, 4, 2));
		$ip = ord($string[0]).".".ord($string[1]).".".ord($string[2]).".".ord($string[3]);
		return array($ip, $port);
	}

	function encodeIP($address) {
		$oct = explode(".", $address[0]);
		$string = Chr($oct[0]).Chr($oct[1]).Chr($oct[2]).Chr($oct[3]);
		$string .= pack("v", (int)($address[1]));
		return $string;
	}

	function formatstring($text) {
		if (strlen($text) == 4 && $text[2] == "\x00") {
			return bin2hex($text);
		} else {
			return var_export($text, true);
		}
	}
}

class ImpSocket {
	var $socket;
	var $address = array("", "");

	function connect($address, $timeout = 3) {
		if ($this->socket = fsockopen("tcp://".$address[0],$address[1],$errno,$errstr)) {
			socket_set_timeout($this->socket, $timeout);
			$this->address = $address;
			return 1;
		} else {
			return 0;
		}
	}

	function recv($length) {
		return fread($this->socket, $length);
	}

	function recv_withlen() {
		$data = "";
		$lengthstr = $this->recv(4);
		if (strlen($lengthstr) == 4) {
			list(, $length) = unpack("L", $lengthstr[3].$lengthstr[2].$lengthstr[1].$lengthstr[0]);
			$data = $this->recv_all($length);
		}
		return $data;
	}

	function recv_all($length) {
		$data = "";
		while(strlen($data) < $length) {
			$chunk = $this->recv($length - strlen($data));
			if ($chunk == "") { // socket connection broken
				break 1;
			}
			$data .= $chunk;
		}
		return $data;
	}

	function send($data) {
		return fwrite($this->socket, $data);
	}

	function send_withlen($data) {
		$lengthstr = pack("L", strlen($data));
		$lengthstr = $lengthstr[3].$lengthstr[2].$lengthstr[1].$lengthstr[0];
       	$totaldata = $lengthstr.$data;
		return fwrite($this->socket, $totaldata);
	}

	function close() {
		return fclose($this->socket);
	}
}
?>