<?php
if(!CONFIG_LOAD) die();

require_once CLASSES."gcf.class.php";
require_once CLASSES."data.class.php";
require_once CLASSES."steam.class.php";

function compare_gcf($new_gcf, $old_gcf) {
	$self = array();

	$old_ver = New GCFClass;
	$old_ver->Load($old_gcf);

	$new_ver = New GCFClass;
	$new_ver->Load($new_gcf);

	$self['updatefiles'] = array();
	$self['blockcount'] = 0;
	$self['gcf_size'] = 0;
	$self['old_ver'] = $old_ver->appversion;
	$self['new_ver'] = $new_ver->appversion;

	if ($self['old_ver'] == $self['new_ver']) {
		foreach($new_ver->manifest['dir_entries2'] as $new_dirid => $new_dir_entry) {
			if ($old_ver->dirmap_entries[$new_dir_entry['i']] == 0) {
				$self['updatefiles'][$new_dirid] = $new_dir_entry;
				$self['blockcount'] += countblocks($new_dir_entry['itemsize']);
			}
		}
	} else {
		foreach($new_ver->manifest['dir_entries2'] as $new_dirid => $new_dir_entry) {
			if (!$new_dir_entry['fileid']) continue;
			if (!$new_dir_entry['dirtype']) continue;
			if ($new_dir_entry['dirtype'] & 0x100) die("File encrypted(in GCF): ".$new_dir_entry['fileid']." ".$new_dir_entry['fullfilename']);

			if (!empty($old_ver->manifest['dir_entries2'][$new_dirid])) {
				$old_dir_entry = $old_ver->manifest['dir_entries2'][$new_dirid];
				if (!$new_dir_entry['fileid']) die();
				if (!$new_dir_entry['dirtype']) die();
				if ($old_dir_entry['dirtype'] & 0x100) {
						die("File encrypted(in archive): ".$old_dir_entry['fileid']." ".$old_dir_entry['fullfilename']);
				}
				if ($new_gcf->checksums['checksums_raw'][$new_dirid] != $old_gcf->checksums['checksums_raw'][$new_dirid]) {
					$self['updatefiles'][$new_dirid] = $new_dir_entry;
					$self['blockcount'] += countblocks($new_dir_entry['itemsize']);
				}
				unset($old_dir_entry);
			} else {
				$self['updatefiles'][$new_dirid] = $new_dir_entry;
				$self['blockcount'] += countblocks($new_dir_entry['itemsize']);
			}
			unset($block);
		}
	}

	$firstblockoffset = 44;
	$firstblockoffset += 32;
	$firstblockoffset += (28 * $self['blockcount']);
	$firstblockoffset += 16;
	$firstblockoffset += (4 * $self['blockcount']);
	$firstblockoffset += ($new_ver->dirmap_index - $new_ver->manifest_index);
	$firstblockoffset += 8;
	$firstblockoffset += ($new_ver->manifest['num_items'] * 4);
	$firstblockoffset += 8;
	$firstblockoffset += $new_ver->checksumsize;
	$firstblockoffset += 24;

	$self['gcf_size'] = $firstblockoffset + ($self['blockcount'] * 8192);

	$old_ver->close();
	$new_ver->close();
	unset($old_ver);
	unset($new_ver);

	return $self;
}

function open_table($title) {
	echo "<table width='100%' cellpadding='0' cellspacing='0'>";
	echo "<tr><td class='panel-left'><img src='images/blank.gif' width='14' height='21' alt='' style='display:block'></td>";
	echo "<td width='100%' class='panel-main'>$title</td>";
	echo "<td class='panel-right'><img src='images/blank.gif' width='14' height='21' alt='' style='display:block'></td>";
	echo "</tr></table>";
	echo "<table width='100%' cellpadding='0' cellspacing='0'><tr>";
	echo "<td class='main-body'>\n";
}

function close_table() {
	echo "</td></tr><tr><td width='100%' height='8'></td></tr></table>";
}

function tablebreak() {
	echo "<table width='100%' cellspacing='0' cellpadding='0'><tr><td height='8'></td></tr></table>\n";
}

function parsesize($size) {
	$kb=1024; $mb=1024*$kb; $gb=1024*$mb; $tb=1024*$gb;
	if (($size==0)) { return " Empty"; }
	elseif ($size<$kb) { return $size." Bytes"; }
	elseif ($size<$mb) { return round($size/$kb,2)." Kb"; }
	elseif ($size<$gb) { return round($size/$mb,2)." Mb"; }
	elseif ($size<$tb) { return round($size/$gb,2)." Gb"; }
	else { return round($size/$tb,2)." Tb"; }
}

function calcbytes($data) {
	$length = strlen($data);
	$count = 0;
	while($length) { $length--; $count += ord($data[$length]); }
	return $count;
}

function countblocks($length, $blocksize = 8192) {
	$count = ($length / $blocksize);
	if (strpos($count, ".")) { $count += 1; }
	return (int)($count);
}

function microtime_float() {
	list($usec, $sec) = explode(" ", microtime());
	return ((float)$usec + (float)$sec);
}

function get_gcf_info($gcf_path) {
	$self = array();
	if ($gcf = fopen($gcf_path, "r")) {
		$header = fread($gcf, 40);
		list(,$self['header_dummy1'],
		$self['header_dummy2'],
		$self['gcfversion'],
		$self['appid'],
		$self['appversion'],
		$self['header_dummy3'],
		$self['header_dummy4'],
		$self['filesize'],
		$self['blocksize'],
		$self['blockcount']) = unpack("L10", $header);
		list(,$self['checksum']) = unpack("L1", fread($gcf, 4));
		fclose($gcf);
	}
	if (($self['gcfversion'] == 6 || $self['gcfversion'] == 1) && calcbytes($header) == $self['checksum']) {
		$self['path'] = $gcf_path;
		$self['filename'] = strtolower(substr(strrchr($gcf_path, "/"),1));
		return $self;
	}
}

$storage = array();
if ($open = @opendir(CACHES_FOLDER)) {
	for($i=0;($file = readdir($open)) != false;$i++){
		$extension = strtolower(substr(strrchr($file, "."),1));
		if ($file != "." && $file != ".." && is_file(CACHES_FOLDER.$file) && ($extension == "gcf" || $extension == "ncf")) {
			$gcf = get_gcf_info(CACHES_FOLDER.$file);
			if (!empty($gcf['gcfversion'])) {
				$storage[$gcf['appid']] = $gcf;
				unset($gcf);
			}
		}
	}
	closedir($open);
}
?>